/*
 I have stripped out unessential things from the Animator and Mover classes.
 I added addMouseListenerToScene and addMouseMotionListenerToScene so that other
 MouseListeners can be added.
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

//--------------------------------------------------------------------
// Animator - Main class extended by the user.
//             Initializes the display, starts animation
//             Has draw method for the user to extend
//--------------------------------------------------------------------

public abstract class Animator extends JApplet implements ComponentListener
{
    private JPanel scene = new JPanel(); // Panel where animation takes place.
    private Mover move;                 // Thread that runs the animation.
    private static Animator anim;    // for static I/O and service methods

    //----------------------------------------------------------------
    // init - initializes display and starts animation thread
    //----------------------------------------------------------------
    public void init() {
        Container window = getContentPane();
        window.add(scene,BorderLayout.CENTER);
        window.validate();   // make sure everything is displayed
        window.addComponentListener(this); // In case window is resized.
        Animator.setAnim(this);         // for static I/O and service methods
        move = new Mover(this, scene);  // create and start the animation thread
        move.start();
    } // end of init method

    // **********************************************************
    //    I added these so that additional mouse listeners can be used.  RLF
    public void addMouseListenerToScene(MouseListener listener)
    {   scene.addMouseListener(listener);   }
    public void addMouseMotionListenerToScene(MouseMotionListener listener)
    {   scene.addMouseMotionListener(listener);     }
    // **********************************************************

    //----------------------------------------------------------------
    // componentResized - adjust animation to fit new window size
    //----------------------------------------------------------------
    public void componentResized(ComponentEvent event) {
        move.resize();
        validate();
    } // end of componentResized method

    //----------------------------------------------------------------
    // unused methods defined to satisfy ComponentListener interface 
    //----------------------------------------------------------------
    public void componentHidden(ComponentEvent event) {}
    public void componentMoved(ComponentEvent event) {}
    public void componentShown(ComponentEvent event) {}

    //----------------------------------------------------------------
    // startup - called before animation begins
    //           should be overridden by user for initialization purposes
    //----------------------------------------------------------------
    public void startup() {}

    //----------------------------------------------------------------
    // draw - draw a static picture in the scene. Called by animator 10 times per second.
    //        Should be overriddend by user.
    //----------------------------------------------------------------
    public void draw(Graphics g) { } // end of draw method

    //----------------------------------------------------------------
    // setAnim - save a reference to the Animator object for static I/O and information methods
    //----------------------------------------------------------------
    public static void setAnim(Animator a) {
        anim = a;
    }

    //----------------------------------------------------------------
    // getSceneWidth - returns the current width of the scene in pixels 
    //----------------------------------------------------------------
    public static int getSceneWidth() {
        return anim.animGetWidth();
    } // end of getSceneWidth method

    private int animGetWidth() {
        Dimension d = scene.getSize();
        return d.width;
    } // end of animGetWidth method

    //----------------------------------------------------------------
    // getSceneHeight - returns the current height of the scene in pixels (an int)
    //----------------------------------------------------------------
    public static int getSceneHeight() {
        return anim.animGetHeight();
    } // end of getSceneHeight method

    private int animGetHeight() {
        Dimension d = scene.getSize();
        return d.height;
    } // end of animGetHeight method

} // end of Animator class

//--------------------------------------------------------------------
//
//--------------------------------------------------------------------

class Mover extends Thread {

    // property variables
    private JPanel scene;
    private Animator sh;
    private int delay = 2;  // Originally 64.
    private Dimension d;
    private Image buffer;
    private Graphics g;
    private Graphics cg;
    private boolean firstTime = true;

    //----------------------------------------------------------------
    // constructor - initialize the animation thread
    // parameters:
    //    s - a reference to the Animator object
    //    cp - a reference to the scene JPanel object
    //----------------------------------------------------------------

    public Mover(Animator s, JPanel cp) {

        // save object references
        sh = s;
        scene = cp;

        // get size info and create Image to draw into
        d = scene.getSize();
        buffer = sh.createImage(d.width, d.height);

        // get Graphics object refs for drawing
        g = buffer.getGraphics();
        cg = scene.getGraphics();

    } // end of constructor

    //----------------------------------------------------------------
    // resize - reinitialize due to the window being resized
    //----------------------------------------------------------------
    public void resize() {
        // get size info and reinitialize Image and Grahics object refs
        d = scene.getSize();
        buffer = sh.createImage(d.width, d.height);
        g = buffer.getGraphics();
        cg = scene.getGraphics();
    } // end of resize method

    //----------------------------------------------------------------
    // run - main animation thread
    //----------------------------------------------------------------

    public void run() {
        // determine current time in order to set proper delay
        long then = new Date().getTime();

        // call user's startup before starting animation
        if (firstTime) {
            sh.startup();
            firstTime = false;
            }

        // repeat drawing forever
        while (true) {

            // delay just long enough to maintain proper speed
            long now = new Date().getTime();
            long actualDelay = delay - (now-then);
            if (actualDelay > 0) {
                try {
                    sleep(actualDelay);
                    }
                catch(InterruptedException e) {}
                }
                then = new Date().getTime();

                // draw user's picture
                paint();
            }

        } // end of run method

    //----------------------------------------------------------------
    // paint - draw user's picture
    //----------------------------------------------------------------
    private void paint() {
        g.setColor(Color.cyan);
        g.fillRect(0,0,d.width,d.height); // draw background
        sh.draw(g);                       // call user's draw method
        cg.drawImage(buffer,0,0,sh);      // draw user's picture
    } // end of paint method

} // end of Mover class